;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Growing Vine Disassembly, by imamelia
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; init routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

!Map16Left    = $0280
!Map16Right   = $0281
!TileMap1     = $C0
!TileMap2     = $E0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "INIT ",pc
       LDY #$00
       LDA $7FAB10,x
       AND #$04
       BEQ +
       INY
+      TYA
       STA $157C,x
       RTL
print "MAIN ",pc
       PHB
       PHK
       PLB
       JSR GrowingVineMain
       PLB
       RTL

GrowingVineMain:
       LDY #$06
       JSR SubOffScreen

       LDA $64              ; level priority bits
       PHA                  ; save for later
       LDA $1540,x          ;
       CMP #$20             ; if this counter is less than 20...
       BCC +                ; don't change the sprite's priority
       LDA #$10             ;
       STA $64              ; set new priority bits
+
       JSR SubVineGFX       ; the original graphics routine was so screwy that I had to rewrite part of it
       PLA                  ;
       STA $64              ; restore original priority bits from before
       LDA $9D              ; if sprites are locked...
       BNE ReturnMain       ; return
       LDA #$F0             ; set the sprite's
       LDY $157C,x
       BNE +
       LDA #$10
+      STA $B6,x            ; Y speed
       JSL $018022          ; update sprite Y position without gravity      
       LDA $1540,x          ; load the counter
       CMP #$20             ; this time, branch if it is greater than or equal to #$20
       BCS SkipStuff        ;
       JSL $019138          ; interact with objects
       LDA $1588,x          ; load sprite blocked status
       BNE SkipStuff        ; branch if the sprite is touching anything  
       LDA $14E0,x          ; sprite Y position (high byte)
       BPL DontEraseSprite  ; don't erase the sprite if it hasn't gone offscreen

SkipStuff:
       LDA $14C8,x          ; if the sprite isn't in its normal status...
       CMP #$08             ; (more like, if the status is *less* than 08)
       BCC SpriteDisappears ; then make it disappear
       LDY $161A,x          ; load sprite index to level table
       CPY #$FF             ; if FF...
       BEQ SpriteDisappears ; then make the sprite disappear
       LDA #$00             ; mark the sprite to be loaded
       STA $1938,y          ; based on its index stored in $161A,x
SpriteDisappears:           ;
       STZ $14C8,x          ;
       RTS

DontEraseSprite:            ;
       LDA $E4,x            ; sprite Y position (low byte)
       AND #$0F             ; we only want the lower 4 bits
       BNE ReturnMain       ; if this is 00, then return
       LDA $E4,x            ; sprite X position (low byte)
       STA $9A              ; this will be the low byte of the vine tile's X position
       LDA $14E0,x          ; sprite X position (high byte)
       STA $9B              ; this will be the high byte of the vine tile's X position
       LDA $D8,x            ; sprite Y position (low byte)
       STA $98              ; block Y position (low byte)
       LDA $14D4,x          ; sprite Y position (high byte)
       STA $99              ; block Y position (high byte)
       REP #$10
       LDY #!Map16Left      ; block to generate:
       LDA $7FAB10,x
       AND #$04
       BEQ +
       LDY #!Map16Right
+      REP #$20
       TYA
       STA $03
       JSR SubSetMap16
       SEP #$30
ReturnMain:                 ;
       RTS

;========================
; Graphics routine
;========================

SubVineGFX:
       JSR GetDrawInfo      ; set up variables for OAM
       LDA $00              ; sprite X displacement
       STA $0300,y          ; first OAM slot
       LDA $01              ; sprite Y displacement
       STA $0301,y          ; second OAM slot
       LDA $14              ; take the sprite frame counter
       LSR #4
       LDA #!TileMap1       ; tile used if bit 4 of the frame counter was clear
       BCC StoreTile        ;
       LDA #!TileMap2       ; tile used if bit 4 of the frame counter was set
StoreTile:                  ;
       STA $0302,y          ; store the tile number      
       LDA $157C,x          ;
       LSR                  ;
       LDA $15F6,x          ; load sprite palette and GFX page info
       BCS NoFlip           ; if the sprite's direction was right ($157C,x and $02=01),
       ORA #$40             ; flip the tile
NoFlip:
       ORA $64              ; add in priority bits
       STA $0303,y          ; store tile properties
       LDY #$02             ; the tiles are 16x16
       TDC
       JSL $01B7B3          ; finish OAM write
       RTS

;========================
; Misc. subroutines
;========================

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  GetDrawInfo:            ;; - Optimized version
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

       LDA $14E0,x
       XBA
       LDA $E4,x
       REP #$20
       SEC
       SBC $1A
       STA $00
       CLC
       ADC #$0040
       CMP #$0180
       SEP #$20
       LDA $01
       BEQ +
       LDA #$01
+      STA $15A0,x
       TDC
       ROL A
       STA $15C4,x
       BNE .Invalid
       LDA $14D4,x
       XBA
       LDA $1662,x
       AND #$20
       BEQ .CheckOnce
.CheckTwice
       LDA $D8,x
       REP #$21
       ADC #$001C
       SEC
       SBC $1C
       SEP #$20
       LDA $14D4,x
       XBA
       BEQ .CheckOnce
       LDA #$02
.CheckOnce
       STA $186C,x
       LDA $D8,x
       REP #$21
       ADC #$000C
       SEC
       SBC $1C
       SEP #$21
       SBC #$0C
       STA $01
       XBA
       BEQ .OnScreenY
       INC $186C,x
.OnScreenY
       LDY $15EA,x
       RTS

.Invalid
       PLA
       PLA
       RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  SubOffScreen:           ;; - Optimized version
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Argument
; Y : type (#$00=X0, #$02=X1 ... #$0E=X7)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
       
       LDA $15A0,x
       ORA $186C,x
       BEQ .Return2
       LDA $167A,x
       AND #$04
       BNE .Return2
       LDA $5B
       LSR A
       BCS .VerticalLevel
       LDA $D8,x
       ADC #$50
       LDA $14D4,x
       ADC #$00
       CMP #$02
       BCS .EraseSprite
       LDA $14E0,x
       XBA
       LDA $E4,x
       REP #$21
       ADC .AddTable,y
       SEC
       SBC $1A
       CMP .CmpTable,y
.Common
       SEP #$20
       BCC .Return2
.EraseSprite
       LDA $14C8,x
       CMP #$08
       BCC .KillSprite
       LDY $161A,x
       CPY #$FF
       BEQ .KillSprite
       LDA $1938,y
       AND #$FE
       STA $1938,y
.KillSprite
       STZ $14C8,x
.Return2
       RTS

.VerticalLevel
       LDA $13
       LSR A
       BCS .CheckY
       LDA $14E0,x
       XBA
       LDA $E4,x
       REP #$21
       ADC.w #$0040
       CMP.w #$0280
       BRA .Common
.CheckY
       LDA $14D4,x
       XBA
       LDA $D8,x
       REP #$20
       SBC $1C
       CLC
       ADC.w #$0070
       CMP.w #$01D0
       BRA .Common

.AddTable
       dw $0040,$0040,$0010,$0070
       dw $0090,$0050,$0080,$FFC0
.CmpTable
       dw $0170,$01E0,$01B0,$01D0
       dw $0230,$01B0,$0220,$0160

SubSetMap16:
       PHP
       REP #$30
       PHY
       PHX
       TAX
       LDA $03
       PHA
       JSR .Sub8034
       PLA
       STA $03
       PLX
       PLY
       PLP
       RTS

.Return18
       PLX
       PLB
       PLP
       RTS

.Sub8034
       PHP
       SEP #$20
       PHB
       LDA #$00
       PHA
       PLB
       REP #$30
       PHX
       LDA $9A
       STA $0C
       LDA $98
       STA $0E
       LDA.w #$0000
       SEP #$20
       LDA $5B
       STA $09
       LDA $1933
       BEQ .NoShift
       LSR $09
.NoShift
       LDY $0E
       LDA $09
       AND #$01
       BEQ .Horiz
       LDA $9B
       STA $00
       LDA $99
       STA $9B
       LDA $00  
       STA $99
       LDY $0C
.Horiz
       CPY #$0200
       BCS .Return18
       LDA $1933
       ASL
       TAX
       LDA $BEA8,x
       STA $65
       LDA $BEA9,x
       STA $66
       STZ $67
       LDA $1925
       ASL
       TAY
       LDA ($65),y
       STA $04
       INY
       LDA ($65),y
       STA $05
       STZ $06
       LDA $9B
       STA $07
       ASL
       CLC
       ADC $07
       TAY
       LDA ($04),y
       STA $6B
       STA $6E
       INY
       LDA ($04),y
       STA $6C
       STA $6F
       LDA #$7E
       STA $6D
       INC
       STA $70
       LDA $09
       AND #$01
       BEQ .NoAnd
       LDA $99
       LSR
       LDA $9B
       AND #$01
       BRA .Label52
.NoAnd
       LDA $9B
       LSR
       LDA $99
       .Label52
       ROL
       ASL #2
       ORA #$20
       STA $04
       CPX.w #$0000
       BEQ .NoAdd
       CLC
       ADC #$10 
       STA $04
.NoAdd
       LDA $98
       AND #$F0
       CLC
       ASL
       ROL
       STA $05
       ROL
       AND #$03
       ORA $04
       STA $06
       LDA $9A
       AND #$F0
       LSR #3
       STA $04
       LDA $05
       AND #$C0
       ORA $04
       STA $07
       REP #$20
       LDA $09
       AND.w #$0001
       BNE .Label51
       LDA $1A
       SEC
       SBC.w #$0080
       TAX
       LDY $1C
       LDA $1933
       BEQ .Label50
       LDX $1E
       LDA $20
       SEC
       SBC.w #$0080
       TAY
       BRA .Label50
.Label51
       LDX $1A
       LDA $1C
       SEC
       SBC.w #$0080
       TAY
       LDA $1933
       BEQ .Label50
       LDA $1E
       SEC
       SBC.w #$0080
       TAX  
       LDY $20
.Label50
       STX $08
       STY $0A
       LDA $98
       AND #$01F0
       STA $04
       LDA $9A
       LSR #4
       AND.w #$000F
       ORA $04
       TAY
       PLA
       SEP #$20
       STA [$6B],y
       XBA
       STA [$6E],y
       XBA
       REP #$20
       ASL A
       TAY
       PHK
       PEA.w .Map16Return-$01
       PEA $804C
       JML $00C0FB|$800000
.Map16Return
       PLB
       PLP
       RTS

